#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2017 CERN
# Author: Javier D. Garcia Lasheras (jgarcia@gl-research.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for the IceStorm flow"""


from __future__ import absolute_import

from .make_syn import ToolSyn
from hdlmake.srcfile import VerilogFile, PCFFile


class ToolIcestorm(ToolSyn):

    """Class providing the interface for IceStorm synthesis"""

    TOOL_INFO = {
        'name': 'IceStorm',
        'id': 'icestorm',
        'windows_bin': None,
        'linux_bin': 'yosys -c',
        'project_ext': ''}

    STANDARD_LIBS = []

    SUPPORTED_FILES = {PCFFile: None}

    HDL_FILES = {VerilogFile: 'read_verilog $(sourcefile)'}

    CLEAN_TARGETS = {'clean': ["$(PROJECT).asc", "$(PROJECT).blif"],
                     'mrproper': ["$(PROJECT).bin"]}

    TCL_CONTROLS = {
        'synthesize': 'yosys -import\n' +
                      'source files.tcl\n' +
                      'synth_ice40 -top $(TOP_MODULE) -blif $(PROJECT).blif',
        'par': 'catch {exec arachne-pnr' +
               ' -d $(SYN_DEVICE)' +
               ' -P $(SYN_PACKAGE)' +
               ' -p $(SOURCES_PCFFile)' +
               ' -o $(PROJECT).asc' +
               ' $(PROJECT).blif}',
        'bitstream': 'catch {exec icepack $(PROJECT).asc $(PROJECT).bin}',
        'install_source': ''}

    def __init__(self):
        super(ToolIcestorm, self).__init__()
        self._tool_info.update(ToolIcestorm.TOOL_INFO)
        self._hdl_files.update(ToolIcestorm.HDL_FILES)
        self._supported_files.update(ToolIcestorm.SUPPORTED_FILES)
        self._standard_libs.extend(ToolIcestorm.STANDARD_LIBS)
        self._clean_targets.update(ToolIcestorm.CLEAN_TARGETS)
        self._tcl_controls.update(ToolIcestorm.TCL_CONTROLS)

    def _makefile_syn_top(self):
        self.manifest_dict["syn_family"] = 'iCE40'
        super(ToolIcestorm, self)._makefile_syn_top()




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the base class for the different code fetchers"""

from __future__ import absolute_import
import os
from hdlmake.util import shell


class Fetcher(object):

    """Base class for the code fetcher objects"""

    def fetch(self, module):
        """Stub method, this must be implemented by the code fetcher"""
        pass

    @staticmethod
    def check_id(path, command):
        """Use the provided command to get the specific ID from
        the repository at path"""
        cur_dir = os.getcwd()
        os.chdir(path)
        identifier = shell.run(command)
        os.chdir(cur_dir)
        return identifier



target = "altera"
action = "synthesis"

syn_device = "ep3c25"
syn_grade = "c6"
syn_package = "f324"
syn_top = "cyclone3_top"
syn_project = "demo"
syn_tool = "quartus"

quartus_preflow = "../../../top/cyclone3_sk/pinout.tcl"
quartus_postmodule = "../../../top/cyclone3_sk/module.tcl"

modules = {
  "local" : [ "../../../top/cyclone3_sk/verilog" ],
}




"""Module providing the synthesis functionality for writing Makefiles"""

from __future__ import absolute_import
import os, sys
import logging
import string

from .makefile import ToolMakefile
from hdlmake.util import shell


def _check_synthesis_manifest(manifest_dict):
    """Check the manifest contains all the keys for a synthesis project"""
    if not manifest_dict["syn_tool"]:
        logging.error(
            "syn_tool variable must be set in the top manifest.")
        sys.exit("Exiting")
    if not manifest_dict["syn_device"]:
        logging.error(
            "syn_device variable must be set in the top manifest.")
        sys.exit("Exiting")
    if not manifest_dict["syn_grade"]:
        logging.error(
            "syn_grade variable must be set in the top manifest.")
        sys.exit("Exiting")
    if not manifest_dict["syn_package"]:
        logging.error(
            "syn_package variable must be set in the top manifest.")
        sys.exit("Exiting")
    if not manifest_dict["syn_top"]:
        logging.error(
            "syn_top variable must be set in the top manifest.")


class ToolSyn(ToolMakefile):

    """Class that provides the synthesis Makefile writing methods and status"""

    def __init__(self):
        super(ToolSyn, self).__init__()

    def write_makefile(self, config, fileset, filename=None):
        """Generate a Makefile for the specific synthesis tool"""
        _check_synthesis_manifest(config)
        self.makefile_setup(config, fileset,
            filename=filename)
        self.makefile_check_tool('syn_path')
        self.makefile_includes()
        self._makefile_syn_top()
        self._makefile_syn_tcl()
        self._makefile_syn_local()
        self._makefile_syn_files()
        self._makefile_syn_command()
        self._makefile_syn_build()
        self._makefile_syn_clean()
        self._makefile_syn_phony()
        logging.info(self._tool_info['name'] + " synthesis makefile generated.")

    def _makefile_syn_top(self):
        """Create the top part of the synthesis Makefile"""
        if shell.check_windows():
            tcl_interpreter = self._tool_info["windows_bin"]
        else:
            tcl_interpreter = self._tool_info["linux_bin"]
        top_parameter = string.Template("""\
TOP_MODULE := ${top_module}
PWD := $$(shell pwd)
PROJECT := ${project_name}
PROJECT_FILE := $$(PROJECT).${project_ext}
TOOL_PATH := ${tool_path}
TCL_INTERPRETER := ${tcl_interpreter}
ifneq ($$(strip $$(TOOL_PATH)),)
TCL_INTERPRETER := $$(TOOL_PATH)/$$(TCL_INTERPRETER)
endif

SYN_FAMILY := ${syn_family}
SYN_DEVICE := ${syn_device}
SYN_PACKAGE := ${syn_package}
SYN_GRADE := ${syn_grade}
""")
        self.writeln(top_parameter.substitute(
            tcl_interpreter=tcl_interpreter,
            project_name=os.path.splitext(
                self.manifest_dict["syn_project"])[0],
            project_ext=self._tool_info["project_ext"],
            syn_family=self.manifest_dict.get("syn_family", ''),
            syn_device=self.manifest_dict["syn_device"],
            syn_package=self.manifest_dict["syn_package"],
            syn_grade=self.manifest_dict["syn_grade"],
            tool_path=self.manifest_dict["syn_path"],
            top_module=self.manifest_dict["syn_top"]))

    def _makefile_syn_tcl(self):
        """Create the Makefile TCL dictionary for the selected tool"""
        command_list = ["create", "open", "save", "close"]
        for command in command_list:
            if command in self._tcl_controls:
                self.writeln('TCL_{1} := {0}'.format(
                    self._tcl_controls[command], command.upper()))
        self.writeln("""\
ifneq ($(wildcard $(PROJECT_FILE)),)
TCL_CREATE := $(TCL_OPEN)
endif""")
        self.writeln()

    def _makefile_syn_files(self):
        """Write the files TCL section of the Makefile"""
        ret = []
        fileset_dict = {}
        sources_list = []
        fileset_dict.update(self._hdl_files)
        fileset_dict.update(self._supported_files)
        for filetype in fileset_dict:
            file_list = []
            for file_aux in self.fileset:
                if isinstance(file_aux, filetype):
                    file_list.append(shell.tclpath(file_aux.rel_path()))
            if not file_list == []:
                ret.append(
                   'SOURCES_{0} := \\\n'
                   '{1}\n'.format(filetype.__name__,
                               ' \\\n'.join(file_list)))
                if not fileset_dict[filetype] is None:
                    sources_list.append(filetype)
        self.writeln('\n'.join(ret))
        self.writeln('files.tcl:')
        if "files" in self._tcl_controls:
            echo_command = '\t\t@echo {0} >> $@'
            tcl_command = []
            for command in self._tcl_controls["files"].split('\n'):
                tcl_command.append(echo_command.format(command))
            command_string = "\n".join(tcl_command)
            if shell.check_windows():
                command_string = command_string.replace("'", "")
            self.writeln(command_string)
        for filetype in sources_list:
            filetype_string = ('\t\t@$(foreach sourcefile,'
                ' $(SOURCES_{0}), echo "{1}" >> $@ &)'.format(
                filetype.__name__, fileset_dict[filetype]))
            if shell.check_windows():
                filetype_string = filetype_string.replace(
                    '"', '')
            self.writeln(filetype_string)
        self.writeln()

    def _makefile_syn_local(self):
        """Generic method to write the synthesis Makefile local target"""
        self.writeln("#target for performing local synthesis\n"
                     "all: bitstream\n")

    def _makefile_syn_build(self):
        """Generate the synthesis Makefile targets for handling design build"""
        stage_previous = "files.tcl"
        stage_list = ["project", "synthesize", "translate",
                      "map", "par", "bitstream"]
        for stage in stage_list:
            if stage in self._tcl_controls:
                echo_command = '\t\techo {0} >> $@'
                tcl_command = []
                for command in self._tcl_controls[stage].split('\n'):
                    tcl_command.append(echo_command.format(command))
                command_string = "\n".join(tcl_command)
                if shell.check_windows():
                    command_string = command_string.replace(
                        "'", "")
                self.writeln("""\
{0}.tcl:
{3}

{0}: {1} {0}.tcl
\t\t$(SYN_PRE_{2}_CMD)
\t\t$(TCL_INTERPRETER) $@.tcl
\t\t$(SYN_POST_{2}_CMD)
\t\t{4} $@
""".format(stage, stage_previous, stage.upper(),
           command_string, shell.touch_command()))
                stage_previous = stage

    def _makefile_syn_command(self):
        """Create the Makefile targets for user defined commands"""
        stage_list = ["project", "synthesize", "translate",
                      "map", "par", "bitstream"]
        for stage in stage_list:
            if stage in self._tcl_controls:
                self.writeln("""\
SYN_PRE_{0}_CMD := {1}
SYN_POST_{0}_CMD := {2}
""".format(stage.upper(),
           self.manifest_dict.get("syn_pre_" + stage + "_cmd", ''),
           self.manifest_dict.get("syn_post_" + stage + "_cmd", '')))

    def _makefile_syn_clean(self):
        """Print the Makefile clean target for synthesis"""
        self.makefile_clean()
        self.writeln("\t\t" + shell.del_command() +
                     " project synthesize translate map par bitstream")
        self.writeln("\t\t" + shell.del_command() +
                     " project.tcl synthesize.tcl translate.tcl" +
                     " map.tcl par.tcl bitstream.tcl files.tcl")
        self.writeln()
        self.makefile_mrproper()

    def _makefile_syn_phony(self):
        """Print synthesis PHONY target list to the Makefile"""
        self.writeln(
            ".PHONY: mrproper clean all")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#         Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""This is the Python module providing the container for the HDL Modules"""

from .action import ActionCore, ActionTree


class ModulePool(ActionCore, ActionTree):

    """
    The ModulePool class acts as the container for the HDLMake modules that
    are progressively being added to the design hierarchy.
    """

    def __init__(self, *args):
        ActionCore.__init__(self, *args)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides the common stuff for the different supported actions"""


from __future__ import print_function
from __future__ import absolute_import
import os
import logging
import sys

from hdlmake.tools import load_syn_tool, load_sim_tool
from hdlmake.util import shell
from hdlmake.util.termcolor import colored
from hdlmake import new_dep_solver as dep_solver
from hdlmake.srcfile import SourceFileSet, VHDLFile, VerilogFile, SVFile

def set_logging_level(options):
    """Set the log level and config (A.K.A. log verbosity)"""
    numeric_level = getattr(logging, options.log.upper(), None)
    if not isinstance(numeric_level, int):
        sys.exit('Invalid log level: %s' % options.log)

    if not shell.check_windows() and options.logfile == None:
        logging.basicConfig(
            format=colored(
                "%(levelname)s",
                "yellow") + colored(
                "\t%(filename)s:%(lineno)d: %(funcName)s()\t",
                "blue") + "%(message)s",
            level=numeric_level)
    else:
        logging.basicConfig(
            format="%(levelname)s" +
                   "\t%(filename)s:%(lineno)d: %(funcName)s()\t" +
                   "%(message)s",
            level=numeric_level,
            filename=options.logfile)
    logging.debug(str(options))


class Action(list):

    """This is the base class providing the common Action methods"""

    def __init__(self, options):
        super(Action, self).__init__()
        self.top_module = None
        self.parseable_fileset = SourceFileSet()
        self.privative_fileset = SourceFileSet()
        self._deps_solved = False
        self.options = options
        set_logging_level(options)
        self.new_module(parent=None,
                         url=os.getcwd(),
                         source=None,
                         fetchto=".")
        self.config = self._get_config_dict()
        action = self.config.get("action")
        if action == None:
            self.tool = None            
            self.top_entity = self.config.get("top_module", None)
        elif action == "simulation":
            self.tool = load_sim_tool(self.config.get("sim_tool"))
            if (self.config.get("sim_top") == None and
                    not self.config.get("top_module") == None):
                self.config["sim_top"] = self.config["top_module"]
            self.top_entity = self.config.get("sim_top")
        elif action == "synthesis":
            self.tool = load_syn_tool(self.config.get("syn_tool"))
            if (self.config.get("syn_top") == None and
                    not self.config.get("top_module") == None):
                self.config["syn_top"] = self.config["top_module"]
            self.top_entity = self.config.get("syn_top")
        else:
            logging.error("Unknown requested action: %s", action)
            quit(1)

    def new_module(self, parent, url, source, fetchto):
        """Add new module to the pool.

        This is the only way to add new modules to the pool
        Thanks to it the pool can easily control its content

        NOTE: the first module added to the pool will become the top_module!.
        """
        from hdlmake.module import Module, ModuleArgs
        self._deps_solved = False
        new_module_args = ModuleArgs()
        new_module_args.set_args(parent, url, source, fetchto)
        new_module = Module(new_module_args, self)
        if not self.__contains(new_module):
            self._add(new_module)
            if not self.top_module:
                self.top_module = new_module
                new_module.parse_manifest()
        return new_module

    def _check_manifest_variable_is_set(self, name):
        """Method to check if a specific manifest variable is set"""
        if getattr(self.top_module, name) is None:
            logging.error(
                "Variable %s must be set in the manifest "
                "to perform current action (%s)",
                name, self.__class__.__name__)
            sys.exit("\nExiting")

    def _check_manifest_variable_value(self, name, value):
        """Method to check if a manifest variable is set to a specific value"""
        variable_match = False
        manifest_value = getattr(self.top_module, name)
        if manifest_value == value:
            variable_match = True

        if variable_match is False:
            logging.error(
                "Variable %s must be set in the manifest and equal to '%s'.",
                name, value)
            sys.exit("Exiting")

    def build_complete_file_set(self):
        """Build file set with all the files listed in the complete pool"""
        logging.debug("Begin build complete file set")
        all_manifested_files = SourceFileSet()
        for module in self:
            all_manifested_files.add(module.files)
        logging.debug("End build complete file set")
        return all_manifested_files

    def solve_file_set(self):
        """Build file set with only those files required by the top entity"""
        if not self._deps_solved:
            if self.tool == None:
                dep_solver.solve(self.parseable_fileset)
            else:
                dep_solver.solve(self.parseable_fileset,
                                 self.tool.get_standard_libs())
            self._deps_solved = True
        if self.options.all_files:
            return
        solved_files = SourceFileSet()
        solved_files.add(dep_solver.make_dependency_set(
            self.parseable_fileset, self.top_entity,
            self.config.get("extra_modules")))
        self.parseable_fileset = solved_files

    def build_file_set(self):
        """Initialize the parseable and privative fileset contents"""
        total_files = self.build_complete_file_set()
        for file_aux in total_files:
            if self.tool == None:
                if any(isinstance(file_aux, file_type)
                       for file_type in [VHDLFile, VerilogFile, SVFile]):
                    self.parseable_fileset.add(file_aux)
                else:
                    self.privative_fileset.add(file_aux)
            else:
                if any(isinstance(file_aux, file_type)
                       for file_type in self.tool.get_parseable_files()):
                    self.parseable_fileset.add(file_aux)
                elif any(isinstance(file_aux, file_type)
                       for file_type in self.tool.get_privative_files()):
                    self.privative_fileset.add(file_aux)
                else:
                    logging.debug("File not supported by the tool: %s",
                                  file_aux.path)
        if len(self.privative_fileset) > 0:
            logging.info("Detected %d supported files that are not parseable",
                         len(self.privative_fileset))
        if len(self.parseable_fileset) > 0:
            logging.info("Detected %d supported files that can be parsed",
                         len(self.parseable_fileset))

    def get_top_module(self):
        """Get the Top module from the pool"""
        return self.top_module

    def _get_config_dict(self):
        """Get the combined hierarchical Manifest dictionary from the pool"""
        config_dict = {}
        for mod in self:
            manifest_dict_tmp = mod.manifest_dict
            if not manifest_dict_tmp == None:
                if 'fetchto' in manifest_dict_tmp:
                    manifest_dict_tmp['fetchto'] = os.path.relpath(os.path.join(
                        mod.path,
                        mod.manifest_dict['fetchto']))
                manifest_dict_tmp.update(config_dict)
                config_dict = manifest_dict_tmp
        return config_dict

    def _add(self, new_module):
        """Add the given new module if this is not already in the pool"""
        from hdlmake.module import Module
        if not isinstance(new_module, Module):
            raise RuntimeError("Expecting a Module instance")
        if self.__contains(new_module):
            return False
        if new_module.isfetched:
            for mod in new_module.submodules():
                self._add(mod)
        self.append(new_module)
        return True

    def __contains(self, module):
        """Check if the pool contains the given module by checking the URL"""
        for mod in self:
            if mod.url == module.url:
                return True
        return False

    def __str__(self):
        """Cast the module list as a list of strings"""
        return str([str(m) for m in self])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013, 2014 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing the HDLMake Manifest and its associated parser"""

from __future__ import absolute_import
import os
import logging

from .configparser import ConfigParser


class ManifestParser(ConfigParser):

    """This is the class providing HDLMake Manifest parser capabilities"""

    def __init__(self):
        super(ManifestParser, self).__init__(
            description="Configuration options description")
        general_options = [
            {'name': 'top_module',
             'default': None,
             'help': "Top level HDL entity for synthesis and simulation",
             'type': ''},
            {'name': 'extra_modules',
             'default': None,
             'help': "Extra HDL entities that must be present in the design",
             'type': []},
            {'name': 'include_dirs',
             'default': None,
             'help': "Include dirs for Verilog sources",
             'type': []},
            {'name': 'action',
             'default': '',
             'help': "What is the action that should be taken if "
             "HDLMake is run in auto mode (simulation/synthesis)",
             'type': ''},
            {'name': 'language',
             'default': 'VHDL',
             'help': "Default language to be used by the tool ",
             'type': ''},
            {'name': 'library',
             'default': "work",
             'help': "Destination library for module's VHDL files",
             'type': ""},
            {'name': 'incl_makefiles',
             'default': [],
             'help': "List of .mk files appended to toplevel makefile",
             'type': []},
            {'name': 'files',
             'default': [],
             'help': "List of files from the current module",
             'type': ''},
            {'name': 'modules',
             'default': {},
             'help': "List of local modules",
             'type': {}}]
        self.add_option_list(general_options)
        self.add_delimiter()
        self.add_type('include_dirs', type_new="")
        self.add_type('incl_makefiles', type_new='')
        self.add_type('files', type_new=[])
        self.add_allowed_key('modules', key="svn")
        self.add_allowed_key('modules', key="git")
        self.add_allowed_key('modules', key="gitsm")
        self.add_allowed_key('modules', key="local")
        fetch_options = [
            {'name': 'fetchto',
             'default': None,
             'help': "Destination for fetched modules",
             'type': ''},
            {'name': 'fetch_pre_cmd',
             'default': '',
                        'help': "Command to be executed before fetch",
                        'type': ''},
            {'name': 'fetch_post_cmd',
             'default': '',
                        'help': "Command to be executed after fetch",
                        'type': ''}]
        self.add_option_list(fetch_options)
        self.add_delimiter()
        syn_options = [
            {'name': 'syn_tool',
             'default': None,
             'help': "Tool to be used in the synthesis",
             'type': ''},
            {'name': 'syn_path',
             'default': None,
             'help': "Execution path for the Tool to be used in synthesis",
             'type': ''},
            {'name': 'syn_device',
             'default': None,
             'help': "Target FPGA device",
             'type': ''},
            {'name': 'syn_family',
             'default': None,
             'help': "Target FPGA family",
             'type': ''},
            {'name': 'syn_grade',
             'default': None,
             'help': "Speed grade of target FPGA",
             'type': ''},
            {'name': 'syn_package',
             'default': None,
             'help': "Package variant of target FPGA",
             'type': ''},
            {'name': 'syn_top',
             'default': None,
             'help': "Top level module for synthesis",
             'type': ''},
            {'name': 'syn_project',
             'default': None,
             'help': "Project file (.xise, .ise, .qpf)",
             'type': ''},
            {'name': 'syn_properties',
             'default': None,
             'help': "Synthesis properties",
             'type': []},
            {'name': 'syn_pre_project_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: project",
             'type': ''},
            {'name': 'syn_post_project_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: project",
             'type': ''},
            {'name': 'syn_pre_synthesize_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: synthesize",
             'type': ''},
            {'name': 'syn_post_synthesize_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: synthesize",
             'type': ''},
            {'name': 'syn_pre_translate_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: translate",
             'type': ''},
            {'name': 'syn_post_translate_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: translate",
             'type': ''},
            {'name': 'syn_pre_map_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: map",
             'type': ''},
            {'name': 'syn_post_map_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: map",
             'type': ''},
            {'name': 'syn_pre_par_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: par",
             'type': ''},
            {'name': 'syn_post_par_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: par",
             'type': ''},
            {'name': 'syn_pre_bitstream_cmd',
             'default': '',
             'help': "Command to be executed before synthesis: bitstream",
             'type': ''},
            {'name': 'syn_post_bitstream_cmd',
             'default': '',
             'help': "Command to be executed after synthesis: bitstream",
             'type': ''}]
        self.add_option_list(syn_options)
        self.add_delimiter()
        quartus_options = [
            {'name': 'quartus_preflow',
             'default': None,
             'help': "Quartus pre-flow script file",
             'type': ''},
            {'name': 'quartus_postmodule',
             'default': None,
             'help': "Quartus post-module script file",
             'type': ''},
            {'name': 'quartus_postflow',
             'default': None,
             'help': "Quartus post-flow script file",
             'type': ''}]
        self.add_option_list(quartus_options)
        self.add_delimiter()
        sim_options = [
            {'name': 'sim_top',
             'default': None,
             'help': "Top level module for simulation",
             'type': ''},
            {'name': 'sim_tool',
             'default': None,
             'help': "Simulation tool to be used (e.g. isim, vsim, iverilog)",
             'type': ''},
            {'name': 'sim_path',
             'default': None,
             'help': "Execution path for the Tool to be used in simulation",
             'type': ''},
            {'name': 'sim_pre_cmd',
             'default': None,
             'help': "Command to be executed before simulation",
             'type': ''},
            {'name': 'sim_post_cmd',
             'default': None,
             'help': "Command to be executed after simulation",
             'type': ''}]
        self.add_option_list(sim_options)
        self.add_delimiter()
        modelsim_options = [
            {'name': 'modelsim_ini_path',
             'default': None,
             'help': "Directory containing a custom Modelsim .ini file",
             'type': ''},
            {'name': 'vsim_opt',
             'default': "",
             'help': "Additional options for vsim",
             'type': ''},
            {'name': 'vcom_opt',
             'default': "",
             'help': "Additional options for vcom",
             'type': ''},
            {'name': 'vlog_opt',
             'default': "",
             'help': "Additional options for vlog",
             'type': ''},
            {'name': 'vmap_opt',
             'default': "",
             'help': "Additional options for vmap",
             'type': ''}]
        self.add_option_list(modelsim_options)
        self.add_delimiter()
        self.add_option(
            'iverilog_opt',
            default="",
            help="Additional options for IVerilog",
            type='')
        self.add_delimiter()
        self.add_option(
            'ghdl_opt',
            default="",
            help="Additional options for GHDL",
            type='')
        self.add_delimiter()

    def add_option_list(self, option_list):
        """Add to the parser a list with the options and their keys"""
        for option in option_list:
            self.add_option(option["name"],
                            default=option["default"],
                            help=option["help"],
                            type=option["type"])

    def add_manifest(self, path):
        """Add to configuration the Manifest at directory (path) if exists"""
        def _search_for_manifest(path):
            """
            Look for manifest in the given folder and create a Manifest object
            """
            logging.debug("Looking for manifest in " + path)
            dir_files = os.listdir(path)
            if "manifest.py" in dir_files and "Manifest.py" in dir_files:
                logging.error(
                    "Both manifest.py and Manifest.py" +
                    "found in the module directory: %s",
                    path)
                quit(1)
            for filename in dir_files:
                if filename == "manifest.py" or filename == "Manifest.py":
                    if not os.path.isdir(filename):
                        logging.debug("Found manifest for module %s: %s",
                                      path, filename)
                        path_aux = os.path.join(path, filename)
                        if not isinstance(path_aux, str):
                            raise ValueError("Path must be an instance of str")
                        #if not path_mod.is_abs_path(path_aux):
                        #    raise ValueError(
                        #        "Manifest path must be absolute path")
                        return path_aux
            return None
        manifest = _search_for_manifest(path)
        if manifest is None:
            logging.error("No manifest found in path: %s", path)
            quit(1)
        else:
            logging.debug("Parse manifest in: %s", manifest)
            return self.add_config_file(manifest)

    def print_help(self):
        """Print the help for the Manifest parser object"""
        self.help()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing the parser for HDLMake Manifest.py files"""

from __future__ import print_function
from __future__ import absolute_import
import logging
import os
import sys
if not sys.version[0] is "2":
    from io import StringIO
else:
    from StringIO import StringIO
import contextlib


@contextlib.contextmanager
def stdout_io(stdout=None):
    """This is a function used to grab the stdout from
    the executed Manifest.py files"""
    old = sys.stdout
    if stdout is None:
        stdout = StringIO()
    sys.stdout = stdout
    yield stdout
    sys.stdout = old


class ConfigParser(object):

    """Class for parsing python configuration files

    Case1: Normal usage
    >>> f = open("test.py", "w")
    >>> f.write('modules = {"local":"/path/to/local", "svn":"path/to/svn"}; ')
    >>> f.write('fetchto = ".."' )
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("modules", type={})
    >>> p.add_option("fetchto", type='')
    >>> p.add_config_file("test.py")
    >>> p.parse()
    {'modules': {'svn': 'path/to/svn', 'local': '/path/to/local'}, \
'fetchto': '..'}

    Case2: Default value and lack of a variable
    >>> f = open("test.py", "w")
    >>> f.write('a="123"')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("a", type='')
    >>> p.add_option("b", type='', default='borsuk')
    >>> p.add_config_file("test.py")
    >>> p.parse()
    {'a': '123', 'b': 'borsuk'}

    Case3: Multiple types for a variable
    >>> f = open("test.py", "w")
    >>> f.write('a=[1,2,3]')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("a", type=1, default=12)
    >>> p.add_type("a", type=[])
    >>> p.add_config_file("test.py")
    >>> p.parse()
    {'a': [1, 2, 3]}

    Case4: Unrecognized options
    >>> f = open("test.py", "w")
    >>> f.write('a = 123')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("b", type='')
    >>> p.add_config_file("test.py")
    >>> p.parse()
    Traceback (most recent call last):
        File "<stdin>", line 1, in <module>
        File "configparser.py", line 107, in parse
        raise NameError("Unrecognized option: " + key)
    NameError: Unrecognized option: a

    Case5: Invalid parameter type
    >>> f = open("test.py","w")
    >>> f.write('a="123"')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("a", type=0)
    >>> p.add_config_file("test.py")
    >>> p.parse()
    Traceback (most recent call last):
        File "<stdin>", line 1, in <module>
        File "configparser.py", line 110, in parse
        raise RuntimeError("Given option: "+str(type(val))+" doesn't match \
specified types:"+str(opt.types))
    RuntimeError: Given option: <type 'str'> doesn't match specified \
types:[<type 'int'>]

    Case6:
    >>> f = open("test.py","w")
    >>> f.write('a={"zupa":1}')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("a", type={})
    >>> p.add_allowed_key("a", "zupa")
    >>> p.add_config_file("test.py")
    >>> p.parse()
    {'a': {'zupa': 1}}

    Case7
    >>> f = open("test.py","w")
    >>> f.write('a={"kot":1}')
    >>> f.close()
    >>> p = ConfigParser()
    >>> p.add_option("a", type={})
    >>> p.add_allowed_key("a", "kniaz")
    >>> p.add_config_file("test.py")
    >>> p.parse()
    Traceback (most recent call last):
        File "<stdin>", line 1, in <module>
        File "configparser.py", line 184, in parse
        raise RuntimeError("Encountered unallowed key: " +key+ " for options \
'"+opt_name+"'")
    RuntimeError: Encountered unallowed key: kot for options 'a'

    Cleanup:
    >>> import os
    >>> os.remove("test.py")
    """

    class Option(object):

        """This subclass provides instances acting as a convenient storage
        for Manifest.py options"""

        def __init__(self, name, **others):
            self.name = name
            self.keys = []
            self.types = []
            self.help = ""
            self.arbitrary_code = ""
            self.global_code = ""

            for key in others:
                if key == "help":
                    self.help = others["help"]
                elif key == "default":
                    self.default = others["default"]
                elif key == "type":
                    self.add_type(type_obj=others["type"])
                else:
                    raise ValueError("Option not recognized: " + key)

        def add_type(self, type_obj):
            """Add a new supported type for the option's value"""
            self.types.append(type(type_obj))

        def add_key(self, key):
            """Add a new dict key. Note that this is only allowed when
            the option's value is a dict!!"""
            if not isinstance(key, str):
                raise ValueError("Allowed key must be a string")
            try:
                self.keys.append(key)
            except AttributeError:
                if type(dict()) not in self.types:
                    raise RuntimeError(
                        "Allowing a key makes sense for dictionaries only")
                self.keys = [key]
            self.keys.append(key)

    def __init__(self, description=None):
        if description is not None:
            if not isinstance(description, str):
                raise ValueError("Description should be a string!")
        self.description = description
        self.options = []
        self.prefix_code = ""
        self.sufix_code = ""
        self.config_file = None

    def __setitem__(self, name, value):
        if name in self.__names():
            for option_x in self.options:
                if option_x.name == name:
                    option_x = value
        else:
            self.options.append(value)

    def __getitem__(self, name):
        if name in self.__names():
            return [x for x in self.options
                    if x is not None and x.name == name][0]
        else:
            raise RuntimeError("No such option as " + str(name))

    def help(self):
        """A method that prints the Manifest help to Host O.S. stdout"""
        print("Variables with special meaning for Hdlmake:")
        for opt in self.options:
            if opt is None:
                print("")
                continue

            line = '  {0:15}; {1:29}; {2:45}{3}{4:10}'
            try:
                tmp_def = opt.default
                if tmp_def == "":
                    tmp_def = '""'
                line = line.format(
                    opt.name,
                    str(opt.types),
                    opt.help,
                    ', default=',
                    tmp_def)
            except AttributeError:  # no default value
                line = line.format(opt.name, str(opt.types), opt.help, "", "")
            print(line)

    def add_option(self, name, **others):
        """Add a new Option object and add it to the parser's option list"""
        if name in self.__names():
            raise ValueError("Option already added: " + name)
        self.options.append(ConfigParser.Option(name, **others))

    def add_type(self, name, type_new):
        """Grab the specified option from parser's list and add a new type"""
        if name not in self.__names():
            raise RuntimeError("Can't add type to a non-existing option")
        self[name].add_type(type_new)

    def add_delimiter(self):
        """Append an empty element in the parser's options list"""
        self.options.append(None)

    def add_allowed_key(self, name, key):
        """Grab the specified option from parser's list and add a new dict key.
        Note that this is only allowed when the option's value is a dict!!"""
        if not isinstance(key, str):
            raise ValueError("Allowed key must be a string")
        try:
            self[name].allowed_keys.append(key)
        except AttributeError:
            if type(dict()) not in self[name].types:
                raise RuntimeError(
                    "Allowing a key makes sense for dictionaries only")
            self[name].allowed_keys = [key]
        self[name].allowed_keys.append(key)

        self[name].add_key(key)

    def add_config_file(self, config_file):
        """Add the Manifest to be processed by the parser"""
        if self.config_file is not None:
            raise RuntimeError("Config file should be added only once")
        if not os.path.exists(config_file):
            raise RuntimeError("Config file doesn't exists: " + config_file)
        self.config_file = config_file
        return True

    def add_prefix_code(self, code):
        """Add the arbitrary Python to be executed just before the Manifest"""
        self.prefix_code += code + '\n'

    def add_sufix_code(self, code):
        """Add the arbitrary Python to be executed just after the Manifest"""
        self.sufix_code += code + '\n'

    def __names(self):
        """A method that returns a list containing the name for every non
        empty object in the parser's option instance list"""
        return [o.name for o in self.options if o is not None]

    def __parser_runner(self, content, extra_context):
        """method that acts as an 'exec' wraper to run the Python code"""
        options = {}
        try:
            with stdout_io() as stdout_aux:
                root_path = os.getcwd()
                exec_path = os.path.dirname(self.config_file)
                os.chdir(exec_path)
                exec(content, extra_context, options)
                os.chdir(root_path)
            printed = stdout_aux.getvalue()
            if len(printed) > 0:
                logging.info(
                    "The manifest inside " +
                    self.config_file +
                    " tried to print something:")
                for line in printed.split('\n'):
                    print("> " + line)
        except SyntaxError as error_syntax:
            logging.error("Invalid syntax in the manifest file " +
                          self.config_file + ":\n" + str(error_syntax))
            logging.error(content)
            quit(1)
        except SystemExit as error_exit:
            logging.error("Exit requested by the manifest file " +
                          self.config_file + ":\n" + str(error_exit))
            logging.error(content)
            quit(1)
        except:
            logging.error("Encountered unexpected error while parsing " +
                          self.config_file)
            logging.error(content)
            print(str(sys.exc_info()[0]) + ':' + str(sys.exc_info()[1]))
            raise
        return options

    def __read_config_content(self):
        """Load the Manifest.py file content in a local variable and return
        the obtained value as a string"""
        if self.config_file is not None:
            with open(self.config_file, "r") as config_file:
                content = config_file.readlines()
                content = ''.join(content)
        else:
            content = ''
        return content

    def parse(self, extra_context=None):
        """Parse the stored manifest plus arbitrary code"""
        assert isinstance(extra_context, dict) or extra_context is None

        # These HDLMake keys must not be inherited from parent module
        key_purge_list = ["modules", "files", "include_dirs",
                          "inc_makefiles", "library"]
        for key_to_be_deleted in key_purge_list:
            extra_context.pop(key_to_be_deleted, None)
        # Load the Manifest.py file content in a local variable
        content = self.__read_config_content()
        # Now, grab the options coming from Manifest.py plus arbitrary_code:
        # - extra_context as global variables.
        # - options as local variables.
        content = self.prefix_code + '\n' + content + '\n' + self.sufix_code
        options = self.__parser_runner(content, extra_context)
        # Check the options that were defined in the local context
        ret = {}
        for opt_name, val in list(options.items()):
            # Manifest variables starting with __(name) will be ignored,
            # so won't be inherited by the childrens of the module using this
            # parser.
            if opt_name.startswith('__'):
                continue
            # Only if the option is not one of the meaningful set for HDLMake,
            # create a new entry in the dictionary to be returned and pass...
            # we won't check the unknown option, but will pass it to the
            # children modules' Manifest
            if opt_name not in self.__names():
                ret[opt_name] = val
                logging.debug("New variable found: %s (=%s).", opt_name, val)
                continue
            # If we are here, is because this is a meaningful option,
            # e.g. syn_top, modules, files... grab the option instance!
            opt = self[opt_name]
            if type(val) not in opt.types:
                raise RuntimeError(
                    "Given option (%s) doesn't match specified types: (%s)" %
                    (str(type(val)), str(opt.types)))
            ret[opt_name] = val
            # Thi is only for the options of the dictionary class:
            if isinstance(val, type(dict())):
                try:
                    for key in val:
                        if key not in self[opt_name].allowed_keys:
                            raise RuntimeError("Encountered unallowed key: "
                                               "%s for option '%s'" %
                                               (key, opt_name))
                except AttributeError:  # no allowed_keys member - don't check
                    pass
        # Set the default values for those values that were not produced
        # by the Python exec operation.
        #for opt in self.options:
        #    try:
        #        if opt.name not in ret:
        #            ret[opt.name] = opt.default
        #    except AttributeError:  # no default value in the option
        #        pass
        return ret


def _test():
    """This funtion provides an embedded test for the Manifest parser"""
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



"""Module providing the simulation functionality for writing Makefiles"""

from __future__ import absolute_import
import os
import sys
import string
import logging

from .makefile import ToolMakefile
from hdlmake.util import shell
from hdlmake.srcfile import VerilogFile, VHDLFile, SVFile


def _check_simulation_manifest(manifest_dict):
    """Check if the simulation keys are provided by the top manifest"""
    if not manifest_dict["sim_top"]:
        logging.error("sim_top variable must be set in the top manifest.")
        sys.exit("Exiting")
    if not manifest_dict["sim_tool"]:
        logging.error("sim_tool variable must be set in the top manifest.")
        sys.exit("Exiting")


class ToolSim(ToolMakefile):

    """Class that provides the Makefile writing methods and status"""

    def __init__(self):
        super(ToolSim, self).__init__()
        self._simulator_controls = {}

    def write_makefile(self, config, fileset, filename=None):
        """Execute the simulation action"""
        _check_simulation_manifest(config)
        self.makefile_setup(config, fileset, filename=filename)
        self.makefile_check_tool('sim_path')
        self.makefile_includes()
        self._makefile_sim_top()
        self._makefile_sim_options()
        self._makefile_sim_local()
        self._makefile_sim_sources()
        self._makefile_sim_compilation()
        self._makefile_sim_command()
        self._makefile_sim_clean()
        self._makefile_sim_phony()

    def _makefile_sim_top(self):
        """Generic method to write the simulation Makefile top section"""
        top_parameter = string.Template("""\
TOP_MODULE := ${top_module}
PWD := $$(shell pwd)
""")
        self.writeln(top_parameter.substitute(
            top_module=self.manifest_dict["sim_top"]))

    def _makefile_sim_options(self):
        """End stub method to write the simulation Makefile options section"""
        pass

    def _makefile_sim_compilation(self):
        """End stub method to write the simulation Makefile compilation
        section"""
        pass

    def _makefile_sim_local(self):
        """Generic method to write the simulation Makefile local target"""
        self.writeln("#target for performing local simulation\n"
                     "local: sim_pre_cmd simulation sim_post_cmd\n")

    def _makefile_sim_sources(self):
        """Generic method to write the simulation Makefile HDL sources"""
        fileset = self.fileset
        self.write("VERILOG_SRC := ")
        for vlog in fileset.filter(VerilogFile):
            if not vlog.is_include:
                self.writeln(vlog.rel_path() + " \\")
        self.writeln()
        self.write("VERILOG_OBJ := ")
        for vlog in fileset.filter(VerilogFile):
            if vlog.is_include:
                continue
            # make a file compilation indicator (these .dat files are made even
            # if the compilation process fails) and add an ending according
            # to file's extension (.sv and .vhd files may have the same
            # corename and this causes a mess
            self.writeln(
                os.path.join(
                    vlog.library,
                    vlog.purename,
                    "." +
                    vlog.purename +
                    "_" +
                    vlog.extension(
                    )) +
                " \\")
        self.writeln()
        self.write("VHDL_SRC := ")
        for vhdl in fileset.filter(VHDLFile):
            self.write(vhdl.rel_path() + " \\\n")
        self.writeln()
        # list vhdl objects (_primary.dat files)
        self.write("VHDL_OBJ := ")
        for vhdl in fileset.filter(VHDLFile):
            # file compilation indicator (important: add _vhd ending)
            self.writeln(
                os.path.join(
                    vhdl.library,
                    vhdl.purename,
                    "." +
                    vhdl.purename +
                    "_" +
                    vhdl.extension(
                    )) +
                " \\")
        self.writeln()

    def _makefile_sim_dep_files(self):
        """Print dummy targets to handle file dependencies"""
        fileset = self.fileset
        for file_aux in fileset:
            if any(isinstance(file_aux, file_type)
                   for file_type in self._hdl_files):
                self.write("%s: %s" % (os.path.join(
                    file_aux.library, file_aux.purename,
                    ".%s_%s" % (file_aux.purename, file_aux.extension())),
                    file_aux.rel_path()))
                # list dependencies, do not include the target file
                for dep_file in [dfile for dfile in file_aux.depends_on
                                 if dfile is not file_aux]:
                    if dep_file in fileset:
                        name = dep_file.purename
                        extension = dep_file.extension()
                        self.write(" \\\n" + os.path.join(
                            dep_file.library, name, ".%s_%s" %
                            (name, extension)))
                    else:
                        # the file is included -> we depend directly on it
                        self.write(" \\\n" + dep_file.rel_path())
                self.writeln()
                is_include = False
                if isinstance(file_aux, VHDLFile):
                    command_key = 'vhdl'
                elif (isinstance(file_aux, VerilogFile) or
                      isinstance(file_aux, SVFile)):
                    is_include = file_aux.is_include
                    command_key = 'vlog'
                if is_include:
                    continue
                self.writeln("\t\t" + self._simulator_controls[command_key])
                self.write("\t\t@" + shell.mkdir_command() + " $(dir $@)")
                self.writeln(" && " + shell.touch_command()  + " $@ \n")
                self.writeln()

    def _makefile_sim_command(self):
        """Generic method to write the simulation Makefile user commands"""
        sim_pre_cmd = self.manifest_dict.get("sim_pre_cmd", '')
        sim_post_cmd = self.manifest_dict.get("sim_post_cmd", '')
        sim_command = string.Template("""# USER SIM COMMANDS
sim_pre_cmd:
\t\t${sim_pre_cmd}
sim_post_cmd:
\t\t${sim_post_cmd}
""")
        self.writeln(sim_command.substitute(sim_pre_cmd=sim_pre_cmd,
                                            sim_post_cmd=sim_post_cmd))

    def _makefile_sim_clean(self):
        """Generic method to write the simulation Makefile user clean target"""
        self.makefile_clean()
        self.makefile_mrproper()

    def _makefile_sim_phony(self):
        """Print simulation PHONY target list to the Makefile"""
        self.writeln(
            ".PHONY: mrproper clean sim_pre_cmd sim_post_cmd simulation")



"""Provides the core functionality for the HDLMake module"""

from __future__ import absolute_import
import os
import sys
import logging

from hdlmake import fetch
from hdlmake.util import path as path_mod


class ModuleConfig(object):

    """This class containt the base properties and methods that
    need to be initialized for a proper behavior"""

    def __init__(self):
        self.source = None
        self.parent = None
        self.url = None
        self.branch = None
        self.revision = None
        self.path = None
        self.isfetched = False

    def process_manifest(self):
        """process_manifest does nothing for ModuleConfig"""
        pass

    def basename(self):
        """Get the basename for the module"""
        if self.source == fetch.SVN:
            return path_mod.svn_basename(self.url)
        else:
            return path_mod.url_basename(self.url)

    def fetchto(self):
        """Get the fetchto folder for the module"""
        return os.path.dirname(self.path)

    def init_config(self, module_args):
        """This initializes the module configuration.
        The function is executed by Module constructor"""
        parent = module_args.parent
        url = module_args.url
        source = module_args.source
        fetchto = module_args.fetchto

        self.source = source
        self.parent = parent

        if self.source != fetch.LOCAL:
            if self.source == fetch.SVN:
                self.url, self.revision = \
                    path_mod.svn_parse(url)
            else:
                self.url, self.branch, self.revision = \
                    path_mod.url_parse(url)
            basename = self.basename()
            path = path_mod.relpath(os.path.abspath(
                os.path.join(fetchto, basename)))

            # Check if the module dir exists and is not empty
            if os.path.exists(path) and os.listdir(path):
                self.path = path
                self.isfetched = True
                logging.debug("Module %s (parent: %s) is fetched.",
                              url, self.parent.path)
            else:
                self.path = path
                self.isfetched = False
                logging.debug("Module %s (parent: %s) is NOT fetched.",
                              url, self.parent.path)
        else:
            self.url, self.branch, self.revision = url, None, None

            if not os.path.exists(url):
                logging.error(
                    "Path to the local module doesn't exist:\n" + url
                    + "\nThis module was instantiated in: " + str(self.parent))
                quit(1)
            self.path = path_mod.relpath(url)
            self.isfetched = True

    def _check_filepath(self, filepath):
        """Check the provided filepath against several conditions"""
        if filepath:
            if path_mod.is_abs_path(filepath):
                logging.warning(
                    "Specified path seems to be an absolute path: " +
                    filepath + "\nOmitting.")
                return False
            filepath = os.path.join(self.path, filepath)
            if not os.path.exists(filepath):
                logging.error(
                    "Path specified in manifest in %s doesn't exist: %s",
                    self.path, filepath)
                sys.exit("Exiting")

            filepath = path_mod.rel2abs(filepath, self.path)
            if os.path.isdir(filepath):
                logging.warning(
                    "Path specified in manifest %s is a directory: %s",
                    self.path, filepath)
        return True

    def _make_list_of_paths(self, list_of_paths):
        """Get a list with only the valid absolute paths from the provided"""
        paths = []
        for filepath in list_of_paths:
            if self._check_filepath(filepath):
                paths.append(path_mod.rel2abs(filepath, self.path))
        return paths


class ModuleCore(ModuleConfig):

    """This is the class providing the module core functionality"""

    def __init__(self):
        # Universal Manifest Properties
        self.library = "work"
        self.action = None
        self.pool = None
        self.top_module = None
        self.manifest_dict = None
        super(ModuleCore, self).__init__()

    def set_pool(self, pool):
        """Set the associated pool for the module instance"""
        self.pool = pool
        self.top_module = pool.get_top_module()

    def process_manifest(self):
        """Method that process the core manifest section"""
        self._process_manifest_universal()
        super(ModuleCore, self).process_manifest()

    def _process_manifest_universal(self):
        """Method processing the universal manifest directives"""
        # if "top_module" in self.manifest_dict:
        #    self.top_module = self.manifest_dict["top_module"]
        # Libraries
        if "library" in self.manifest_dict:
            self.library = self.manifest_dict["library"]
        elif self.parent:
            self.library = self.parent.library

        if "action" in self.manifest_dict:
            self.action = self.manifest_dict["action"].lower()



target = "lattice"
action = "synthesis"

syn_device = "lfxp2-5e"
syn_grade = "-6"
syn_package = "tn144c"
syn_top = "brevia2_top"
syn_project = "demo"
syn_tool = "diamond"

modules = {
  "local" : [ "../../../top/brevia2_dk/verilog" ],
}




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 CERN
# Author: Javier Garcia (jgarcia@gl-research.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""This module provides stuff for cross shell and O.S. support"""

from __future__ import print_function
from __future__ import absolute_import
import os
import sys
import platform
import logging
from subprocess import PIPE, Popen, CalledProcessError


def run(command):
    """Execute a command in the shell and print the output lines as a list"""
    try:
        command_out = Popen(command,
            stdout=PIPE,
            stdin=PIPE,
            stderr=PIPE,
            close_fds=not check_windows(),
            shell=True)
        lines = command_out.stdout.readlines()
        if len(lines) == 0:
            return None
        return lines[0].strip()
    except CalledProcessError as process_error:
        logging.error("Cannot execute the shell command: %s",
            process_error.output)
        quit(1)


def tclpath(path):
    """Convert a O.S. specific path into a TCL friendly one"""
    return path.replace(slash_char(), "/")


def check_windows():
    """Check if we are operating on a Windows filesystem"""
    if platform.system() == 'Windows' or sys.platform == 'cygwin':
        return True
    else:
        return False


def del_command():
    """Get a string with the O.S. specific delete command"""
    if check_windows():
        return "del /s /q /f"
    else:
        return "rm -rf"


def rmdir_command():
    """Get a string with the O.S. specific remove directory command"""
    if check_windows():
        return "rmdir /s /q"
    else:
        return "rm -rf"


def copy_command():
    """Get a string with the O.S. specific copy command"""
    if check_windows():
        return "copy"
    else:
        return "cp"


def mkdir_command():
    """Get a string with the O.S. specific mkdir command"""
    if check_windows():
        return "mkdir"
    else:
        return "mkdir -p"


def touch_command():
    """Get a string with the O.S. specific mkdir command"""
    if check_windows():
        return "type nul >>"
    else:
        return "touch"


def which(filename):
    """Implement the which function and return the paths as a string list"""
    locations = os.environ.get("PATH").split(os.pathsep)
    candidates = []
    for location in locations:
        candidate = os.path.join(location, filename)
        if os.path.isfile(candidate.split()[0]):
            candidates.append(candidate)
    return candidates


def which_cmd():
    """Get a string with the O.S. specific which command"""
    if check_windows():
        return "where"
    else:
        return "which"


def slash_char():
    """Get a string with the O.S. specific path separator"""
    if check_windows():
        return "\\"
    else:
        return "/"


def architecture():
    """Get a string with the O.S. bus width"""
    import struct
    return 64 if struct.calcsize('P') * 8 == 64 else 32



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing the stuff for handling Git repositories"""

from __future__ import absolute_import
import os
from hdlmake.util import path as path_utils
from hdlmake.util import shell
from subprocess import PIPE, Popen
import logging
from .fetcher import Fetcher


class Git(Fetcher):

    """This class provides the Git fetcher instances, that are
    used to fetch and handle Git repositories"""

    def __init__(self):
        self.submodule = False

    @staticmethod
    def get_git_toplevel(module):
        """Get the top level for the Git repository"""
        cur_dir = os.getcwd()
        try:
            os.chdir(path_utils.rel2abs(module.path))
            if not os.path.exists(".gitmodules"):
                return None
            return shell.run("git rev-parse --show-toplevel")
        finally:
            os.chdir(cur_dir)

    @staticmethod
    def get_submodule_commit(submodule_dir):
        """Get the commit for a repository if defined in Git submodules"""
        status_line = shell.run("git submodule status %s" % submodule_dir)
        status_line = status_line.split()
        if len(status_line) == 2 or len(status_line) == 3:
            if status_line[0][0] in ['-', '+', 'U']:
                return status_line[0][1:]
            else:
                return status_line[0]
        else:
            return None

    def fetch(self, module):
        """Get the code from the remote Git repository"""
        fetchto = module.fetchto()
        if not os.path.exists(fetchto):
            os.mkdir(fetchto)
        basename = path_utils.url_basename(module.url)
        mod_path = os.path.join(fetchto, basename)
        if basename.endswith(".git"):
            basename = basename[:-4]  # remove trailing .git
        if not module.isfetched:
            logging.info("Fetching git module %s", mod_path)
            cmd = "(cd {0} && git clone {1})"
            cmd = cmd.format(fetchto, module.url)
            if os.system(cmd) != 0:
                return False
        else:
            logging.info("Updating git module %s", mod_path)
        checkout_id = None
        if module.branch is not None:
            checkout_id = module.branch
            logging.debug("Git branch requested: %s", checkout_id)
        elif module.revision is not None:
            checkout_id = module.revision
            logging.debug("Git commit requested: %s", checkout_id)
        else:
            checkout_id = self.get_submodule_commit(module.path)
            logging.debug("Git submodule commit: %s", checkout_id)
        if checkout_id is not None:
            logging.info("Checking out version %s", checkout_id)
            cmd = "(cd {0} && git checkout {1})"
            cmd = cmd.format(mod_path, checkout_id)
            if os.system(cmd) != 0:
                return False
        if self.submodule and not module.isfetched:
            cmd = ("(cd {0} && git submodule init &&"
                "git submodule update --recursive)")
            cmd = cmd.format(mod_path)
            if os.system(cmd) != 0:
                return False
        module.isfetched = True
        module.path = mod_path
        return True

    @staticmethod
    def check_git_commit(path):
        """Get the revision number for the Git repository at path"""
        git_cmd = 'git log -1 --format="%H" | cut -c1-32'
        return Fetcher.check_id(path, git_cmd)

    @staticmethod
    def get_git_submodules(module):
        """Get a dictionary containing the git submodules
        that are listed in the module's path"""
        submodule_dir = path_utils.rel2abs(module.path)
        if module.isfetched == False:
            logging.debug("Cannot check submodules, module %s is not fetched",
                submodule_dir)
            return {}
        logging.debug("Checking git submodules in %s",
            submodule_dir)
        cur_dir = os.getcwd()
        try:
            os.chdir(submodule_dir)

            if not os.path.exists(".gitmodules"):
                return {}
            config_submodules = {}
            config_content = Popen("git config -f .gitmodules --list",
                                      stdout=PIPE,
                                      stdin=PIPE,
                                      close_fds=not shell.check_windows(),
                                      shell=True)
            config_lines = [line.strip() for line
                            in config_content.stdout.readlines()]
            config_submodule_lines = [line for line in config_lines
                                      if line.startswith("submodule")]
            for line in config_submodule_lines:
                line_split = line.split("=")
                lhs = line_split[0]
                rhs = line_split[1]
                lhs_split = lhs.split(".")
                module_name = '.'.join(lhs_split[1:-1])
                if module_name not in config_submodules:
                    config_submodules[module_name] = {}
                config_submodules[module_name][lhs_split[-1]] = rhs

            if len(list(config_submodules)) > 0:
                logging.info("Found git submodules in %s: %s",
                    module.path, str(config_submodules))
        finally:
            os.chdir(cur_dir)
        return config_submodules


class GitSM(Git):

    def __init__(self):
        self.submodule = True



action = "simulation"
sim_tool = "iverilog"
sim_top = "counter_tb"

sim_post_cmd = "vvp counter_tb.vvp; gtkwave counter_tb.vcd"

modules = {
  "local" : [ "../../../testbench/counter_tb/verilog" ],
}




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for Microsemi Libero IDE synthesis"""


from __future__ import absolute_import
from .make_syn import ToolSyn
from hdlmake.srcfile import VHDLFile, VerilogFile, SDCFile, PDCFile


class ToolLibero(ToolSyn):

    """Class providing the interface for Microsemi Libero IDE synthesis"""

    TOOL_INFO = {
        'name': 'Libero',
        'id': 'libero',
        'windows_bin': 'libero.exe SCRIPT:',
        'linux_bin': 'libero SCRIPT:',
        'project_ext': 'prjx'}

    STANDARD_LIBS = ['ieee', 'std']

    _LIBERO_SOURCE = 'create_links {0} $(sourcefile)'

    SUPPORTED_FILES = {
        SDCFile: _LIBERO_SOURCE.format('-sdc'),
        PDCFile: _LIBERO_SOURCE.format('-pdc')}

    HDL_FILES = {
        VHDLFile: _LIBERO_SOURCE.format('-hdl_source'),
        VerilogFile: _LIBERO_SOURCE.format('-hdl_source')}

    CLEAN_TARGETS = {'clean': ["$(PROJECT)"],
                     'mrproper': ["*.pdb", "*.stp"]}

    TCL_CONTROLS = {
        'create': 'new_project -location {{./{0}}} -name {{{0}}}'
                  ' -hdl {{VHDL}} -family {{ProASIC3}} -die {{{1}}}'
                  ' -package {{{2}}} -speed {{{3}}} -die_voltage {{1.5}}',
        'open': 'open_project -file {$(PROJECT)/$(PROJECT_FILE)}',
        'save': 'save_project',
        'close': 'close_project',
        'project': '$(TCL_CREATE)\n'
                   'source files.tcl\n'
                   '{0}\n'
                   '$(TCL_SAVE)\n'
                   '$(TCL_CLOSE)',
        'bitstream': '$(TCL_OPEN)\n'
                     'update_and_run_tool'
                     ' -name {GENERATEPROGRAMMINGDATA}\n'
                     '$(TCL_SAVE)\n'
                     '$(TCL_CLOSE)',
        'install_source': '$(PROJECT)/designer/impl1/$(SYN_TOP).pdb'}

    def __init__(self):
        super(ToolLibero, self).__init__()
        self._tool_info.update(ToolLibero.TOOL_INFO)
        self._hdl_files.update(ToolLibero.HDL_FILES)
        self._supported_files.update(ToolLibero.SUPPORTED_FILES)
        self._standard_libs.extend(ToolLibero.STANDARD_LIBS)
        self._clean_targets.update(ToolLibero.CLEAN_TARGETS)
        self._tcl_controls.update(ToolLibero.TCL_CONTROLS)

    def _makefile_syn_tcl(self):
        """Create a Libero synthesis project by TCL"""
        syn_project = self.manifest_dict["syn_project"]
        syn_device = self.manifest_dict["syn_device"]
        syn_grade = self.manifest_dict["syn_grade"]
        syn_package = self.manifest_dict["syn_package"]
        create_tmp = self._tcl_controls["create"]
        self._tcl_controls["create"] = create_tmp.format(syn_project,
                                                         syn_device.upper(),
                                                         syn_package.upper(),
                                                         syn_grade)
        project_tmp = self._tcl_controls["project"]
        synthesis_constraints = []
        compilation_constraints = []
        ret = []
        # First stage: linking files
        for file_aux in self.fileset:
            if isinstance(file_aux, SDCFile):
                synthesis_constraints.append(file_aux)
                compilation_constraints.append(file_aux)
            elif isinstance(file_aux, PDCFile):
                compilation_constraints.append(file_aux)
            else:
                continue
        # Second stage: Organizing / activating synthesis constraints (the top
        # module needs to be present!)
        if synthesis_constraints:
            line = 'organize_tool_files -tool {SYNTHESIZE} '
            for file_aux in synthesis_constraints:
                line = line + '-file {' + file_aux.rel_path() + '} '
            line = line + \
                '-module {$(TOP_MODULE)::work} -input_type {constraint}'
            ret.append(line)
        # Third stage: Organizing / activating compilation constraints (the top
        # module needs to be present!)
        if compilation_constraints:
            line = 'organize_tool_files -tool {COMPILE} '
            for file_aux in compilation_constraints:
                line = line + '-file {' + file_aux.rel_path() + '} '
            line = line + \
                '-module {$(TOP_MODULE)::work} -input_type {constraint}'
            ret.append(line)
        # Fourth stage: set root/top module
        line = 'set_root -module {$(TOP_MODULE)::work}'
        ret.append(line)
        self._tcl_controls['project'] = project_tmp.format('\n'.join(ret))
        super(ToolLibero, self)._makefile_syn_tcl()



import ez_setup
ez_setup.use_setuptools()

from setuptools import (setup, find_packages)

exec(open('hdlmake/_version.py').read())

try:
    __version__
except Exception:
    __version__ = "0.0"  # default if for some reason the exec did not work

setup(
   name="hdlmake",
   version=__version__,
   description="Hdlmake generates multi-purpose makefiles for HDL projects management.",
   author="Javier D. Garcia-Lasheras",
   author_email="hdl-make@ohwr.org",
   license="GPLv3",
   url="http://www.ohwr.org/projects/hdl-make",
   packages=find_packages(),
   entry_points={
      'console_scripts': [
         'hdlmake = hdlmake.__main__:main',
         ], 
   },
   include_package_data=True,  # use MANIFEST.in during install
   classifiers=[
      "Development Status :: 5 - Production/Stable",
      "Environment :: Console",
      "Topic :: Utilities",
      "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
      "Topic :: Software Development :: Build Tools",
    ],
   )



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

"""Module providing graph funtionalities to HDLMake"""

from __future__ import absolute_import
from hdlmake.util import path

import logging

from .action import Action
from hdlmake.dep_file import DepFile


class ActionTree(Action):

    """Class providing methods to create a graph from pool and to analyze it"""

    def __init__(self, *args):
        super(ActionTree, self).__init__(*args)

    def _generate_tree_web(self, hierarchy, top_id):
        """Create a JSON file containing the graph hierarchy from pool"""
        try:
            import json
            import networkx as nx
            from networkx.readwrite import json_graph
        except ImportError as error_import:
            logging.error(error_import)
            quit(1)
        if self.options.mode == 'dfs':
            hierarchy = nx.dfs_tree(hierarchy, top_id)
        elif self.options.mode == 'bfs':
            hierarchy = nx.bfs_tree(hierarchy, top_id, reverse=False)
        data = json_graph.tree_data(hierarchy, root=top_id)
        json_string = json.dumps(data)
        json_file = open("hierarchy.json", "w")
        json_file.write(json_string)
        json_file.close()

    def generate_tree(self):
        """Generate the graph from pool and create the requested outcomes"""
        try:
            import networkx as nx
        except ImportError as error_import:
            logging.error(error_import)
            quit(1)
        unfetched_modules = False
        hierarchy = nx.DiGraph()

        if self.options.mode == "mods":
            for mod_aux in self:
                if not mod_aux.isfetched:
                    unfetched_modules = True
                else:
                    if mod_aux.parent:
                        hierarchy.add_node(mod_aux.path)
                        hierarchy.add_edge(mod_aux.parent.path, mod_aux.path)
                    else:
                        hierarchy.add_node(mod_aux.path)
                        top_id = mod_aux.path
                    if len(mod_aux.files):
                        for file_aux in mod_aux.files:
                            hierarchy.add_edge(mod_aux.path,
                                               path.relpath(file_aux.path))
        elif (self.options.mode == 'dfs' or
              self.options.mode == 'bfs'):


            logging.warning("This is the solved tree")
            #self.top_entity = self.options.top
            self.build_file_set()
            self.solve_file_set()

            from hdlmake.srcfile import SourceFileSet
            from hdlmake.dep_file import DepRelation
            assert isinstance(self.parseable_fileset, SourceFileSet)
            fset = self.parseable_fileset.filter(DepFile)
            # Find the file that provides the named top level entity
            top_rel_vhdl = DepRelation(
                "%s.%s" %
                ("work", self.top_entity), DepRelation.PROVIDE, DepRelation.ENTITY)
            top_rel_vlog = DepRelation(
                "%s.%s" %
                ("work", self.top_entity), DepRelation.PROVIDE, DepRelation.MODULE)
            top_file = None
            for chk_file in fset:
                hierarchy.add_node(path.relpath(chk_file.path))
                for file_required in chk_file.depends_on:
                    hierarchy.add_edge(path.relpath(chk_file.path), path.relpath(file_required.path))
                for rel in chk_file.rels:
                    if (rel == top_rel_vhdl) or (rel == top_rel_vlog):
                        top_file = chk_file
                        top_id = path.relpath(chk_file.path)
            if top_file is None:
                logging.critical('Could not find a top level file that provides the '
                                 'top_module="%s". Continuing with the full file set.',
                                 top_level_entity)

        else:
            logging.error('Unknown tree mode: %s', self.options.mode)
            quit(1)

        if unfetched_modules:
            logging.warning("Some of the modules have not been fetched!")

        self._generate_tree_web(hierarchy, top_id)



action = "simulation"
sim_tool = "iverilog"
sim_top = "counter_tb"

iverilog_opt = "-g2012"

sim_pre_cmd ="echo IMPORTANT, IVerilog always needs a Verilog testbench, no matter if the DUT is written in VHDL!"
sim_post_cmd = "vvp counter_tb.vvp; gtkwave counter_tb.vcd"

files = [
    "../../../modules/counter/vhdl/counter.vhd",
    "../../../testbench/counter_tb/verilog/counter_tb.v",
]




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing support for IVerilog (Icarus Verilog) simulator"""

from __future__ import absolute_import
import string

from .make_sim import ToolSim
from hdlmake.srcfile import VerilogFile, VHDLFile, SVFile


class ToolIVerilog(ToolSim):

    """Class providing the interface for Icarus Verilog simulator"""

    TOOL_INFO = {
        'name': 'Icarus Verilog',
        'id': 'iverilog',
        'windows_bin': None,
        'linux_bin': 'iverilog'}

    STANDARD_LIBS = ['std', 'ieee', 'ieee_proposed', 'vl', 'synopsys']

    HDL_FILES = {VerilogFile: '', VHDLFile: '', SVFile: ''}

    CLEAN_TARGETS = {'clean': ["run.command", "ivl_vhdl_work", "work"],
                     'mrproper': ["*.vcd", "*.vvp"]}

    SIMULATOR_CONTROLS = {'vlog': 'echo $< >> run.command',
                          'vhdl': 'echo $< >> run.command',
                          'compiler': 'iverilog $(IVERILOG_OPT) '
                                      '-s $(TOP_MODULE) '
                                      '-o $(TOP_MODULE).vvp '
                                      '-c run.command'}

    def __init__(self):
        super(ToolIVerilog, self).__init__()
        self._tool_info.update(ToolIVerilog.TOOL_INFO)
        self._hdl_files.update(ToolIVerilog.HDL_FILES)
        self._standard_libs.extend(ToolIVerilog.STANDARD_LIBS)
        self._clean_targets.update(ToolIVerilog.CLEAN_TARGETS)
        self._simulator_controls.update(ToolIVerilog.SIMULATOR_CONTROLS)

    def _makefile_sim_compilation(self):
        """Generate compile simulation Makefile target for IVerilog"""
        self.writeln("simulation: include_dirs $(VERILOG_OBJ) $(VHDL_OBJ)")
        self.writeln("\t\t" + self._simulator_controls['compiler'])
        self.writeln()
        self.writeln("include_dirs:")
        self.writeln("\t\techo \"# IVerilog command file,"
                     " generated by HDLMake\" > run.command")
        self.writeln()
        for inc in self.manifest_dict.get("include_dirs", []):
            self.writeln("\t\techo \"+incdir+" + inc + "\" >> run.command")
        self.writeln('\n')
        self._makefile_sim_dep_files()

    def _makefile_sim_options(self):
        """Print the IVerilog options to the Makefile"""
        iverilog_opt = self.manifest_dict.get("iverilog_opt", '')
        iverilog_string = string.Template(
            """IVERILOG_OPT := ${iverilog_opt}\n""")
        self.writeln(iverilog_string.substitute(
            iverilog_opt=iverilog_opt))



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing common stuff for Modelsim, Vsim... like simulators"""

from __future__ import absolute_import
import os
import string

from .make_sim import ToolSim
from hdlmake.util import shell
from hdlmake.srcfile import VerilogFile, VHDLFile, SVFile
import six


class VsimMakefileWriter(ToolSim):

    """A Makefile writer for simulation suitable for vsim based simulators.

    Currently used by:
      - Modelsim
      - Riviera
    """

    HDL_FILES = {VerilogFile: '', VHDLFile: '', SVFile: ''}

    def __init__(self):
        super(VsimMakefileWriter, self).__init__()
        # These are variables that will be set in the makefile
        # The key is the variable name, and the value is the variable value
        self.custom_variables = {}
        # Additional sim dependencies (e.g. modelsim.ini)
        self.additional_deps = []
        # These are files copied into your working directory by a make rule
        # The key is the filename, the value is the file source path
        self.copy_rules = {}
        self._hdl_files.update(VsimMakefileWriter.HDL_FILES)

    def _makefile_sim_options(self):
        """Print the vsim options to the Makefile"""
        def __get_rid_of_vsim_incdirs(vlog_opt=""):
            """Parse the VLOG options and purge the included dirs"""
            if not vlog_opt:
                vlog_opt = ""
            vlogs = vlog_opt.split(' ')
            ret = []
            for vlog_aux in vlogs:
                if not vlog_aux.startswith("+incdir+"):
                    ret.append(vlog_aux)
            return ' '.join(ret)
        vcom_flags = "-quiet " + self.manifest_dict.get("vcom_opt", '')
        vsim_flags = "" + self.manifest_dict.get("vsim_opt", '')
        vlog_flags = "-quiet " + __get_rid_of_vsim_incdirs(
            self.manifest_dict.get("vlog_opt", ''))
        vmap_flags = "" + self.manifest_dict.get("vmap_opt", '')
        for var, value in six.iteritems(self.custom_variables):
            self.writeln("%s := %s" % (var, value))
        self.writeln()
        self.writeln("VCOM_FLAGS := %s" % vcom_flags)
        self.writeln("VSIM_FLAGS := %s" % vsim_flags)
        self.writeln("VLOG_FLAGS := %s" % vlog_flags)
        self.writeln("VMAP_FLAGS := %s" % vmap_flags)

    def _makefile_sim_compilation(self):
        """Write a properly formatted Makefile for the simulator.
        The Makefile format is shared, but flags, dependencies, clean rules,
        etc are defined by the specific tool.
        """
        def __create_copy_rule(name, src):
            """Get a Makefile rule named name, which depends on src,
            copying it to the local directory."""
            rule = """%s: %s
\t\t%s $< . 2>&1
""" % (name, src, shell.copy_command())
            return rule
        fileset = self.fileset
        if self.manifest_dict.get("include_dirs") == None:
            self.writeln("INCLUDE_DIRS :=")
        else:
            self.writeln("INCLUDE_DIRS := +incdir+%s" %
                ('+'.join(self.manifest_dict.get("include_dirs"))))
        libs = set(f.library for f in fileset)
        self.write('LIBS := ')
        self.write(' '.join(libs))
        self.write('\n')
        # tell how to make libraries
        self.write('LIB_IND := ')
        self.write(' '.join([lib + shell.slash_char() +
                   "." + lib for lib in libs]))
        self.write('\n')
        self.writeln()
        self.writeln(
            "simulation: %s $(LIB_IND) $(VERILOG_OBJ) $(VHDL_OBJ)" %
            (' '.join(self.additional_deps)),)
        self.writeln("$(VERILOG_OBJ) : " + ' '.join(self.additional_deps))
        self.writeln(
            "$(VHDL_OBJ): $(LIB_IND) " +
            ' '.join(
                self.additional_deps))
        self.writeln()
        for filename, filesource in six.iteritems(self.copy_rules):
            self.write(__create_copy_rule(filename, filesource))
        for lib in libs:
            self.write(lib + shell.slash_char() + "." + lib + ":\n")
            vmap_command = "vmap $(VMAP_FLAGS)"
            self.write(' '.join(["\t(vlib", lib, "&&", vmap_command, lib, "&&",
                shell.touch_command(), lib + shell.slash_char() +
                "." + lib, ")"]))
            self.write(' '.join(["||", shell.del_command(), lib, "\n"]))
            self.write('\n\n')
        # rules for all _primary.dat files for sv
        for vlog in fileset.filter(VerilogFile):
            if vlog.is_include:
              continue
            self.write("%s: %s" % (os.path.join(
                vlog.library, vlog.purename,
                ".%s_%s" % (vlog.purename, vlog.extension())),
                vlog.rel_path()))
            # list dependencies, do not include the target file
            for dep_file in [dfile for dfile
                             in vlog.depends_on if dfile is not vlog]:
                if dep_file in fileset and not dep_file.is_include:
                    name = dep_file.purename
                    extension = dep_file.extension()
                    self.write(" \\\n" + os.path.join(
                        dep_file.library, name, ".%s_%s" % (name, extension)))
                else:  # the file is included -> we depend directly on the file
                    self.write(" \\\n" + dep_file.rel_path())
            self.writeln()
            compile_template = string.Template(
                "\t\tvlog -work ${library} $$(VLOG_FLAGS) "
                "${sv_option} $${INCLUDE_DIRS} $$<")
            compile_line = compile_template.substitute(
                library=vlog.library, sv_option="-sv"
                if isinstance(vlog, SVFile) else "")
            self.writeln(compile_line)
            self.write("\t\t@" + shell.mkdir_command() + " $(dir $@)")
            self.writeln(" && " + shell.touch_command() + " $@ \n\n")
            self.writeln()
        # list rules for all _primary.dat files for vhdl
        for vhdl in fileset.filter(VHDLFile):
            lib = vhdl.library
            purename = vhdl.purename
            # each .dat depends on corresponding .vhd file
            self.write("%s: %s" % (os.path.join(
                lib, purename, "." + purename + "_" + vhdl.extension()),
                vhdl.rel_path()))
            # list dependencies, do not include the target file
            for dep_file in [dfile for dfile in vhdl.depends_on
                             if dfile is not vhdl]:
                if dep_file in fileset:
                    name = dep_file.purename
                    extension = dep_file.extension()
                    self.write(" \\\n" + os.path.join(dep_file.library,
                               name, ".%s_%s" % (name, extension)))
                else:
                    self.write(" \\\n" + dep_file.rel_path())
            self.writeln()
            self.writeln(' '.join(["\t\tvcom $(VCOM_FLAGS)",
                         "-work", lib, "$< "]))
            self.writeln("\t\t@" + shell.mkdir_command() +
                " $(dir $@) && " + shell.touch_command() + " $@ \n\n")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.

# Note that the order here is important. The constants must be
# defined first! If they are not, there will likely be an obscure error
# when doing the imports within the imports that come afterwards.

"""Module providing the constants required for the fetch process"""

GIT = 1
GITSM = 2
SVN = 3
LOCAL = 4



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013 - 2015 CERN
# Author: Pawel Szostek (pawel.szostek@cern.ch)
# Multi-tool support by Javier D. Garcia-Lasheras (javier@garcialasheras.com)
#
# This file is part of Hdlmake.
#
# Hdlmake is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hdlmake is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hdlmake.  If not, see <http://www.gnu.org/licenses/>.
#

"""Module providing generic support for Xilinx synthesis tools"""


from __future__ import absolute_import
from .make_syn import ToolSyn
from hdlmake.srcfile import VHDLFile, VerilogFile, SVFile, TCLFile
import logging


class ToolXilinx(ToolSyn):

    """Class providing the interface for Xilinx Vivado synthesis"""

    _XILINX_SOURCE = (
        "add_files -norecurse $(sourcefile); "
        "set_property IS_GLOBAL_INCLUDE 1 [get_files $(sourcefile)]")

    HDL_FILES = {
        VHDLFile: _XILINX_SOURCE,
        VerilogFile: _XILINX_SOURCE,
        SVFile: _XILINX_SOURCE}

    SUPPORTED_FILES = {TCLFile: 'source $(sourcefile)'}

    CLEAN_TARGETS = {'mrproper': ["*.bit", "*.bin"]}

    _XILINX_RUN = '''\
$(TCL_OPEN)
{1}
reset_run {0}
launch_runs {0}
wait_on_run {0}
set result [get_property STATUS [get_runs {0}]]
set keyword [lindex [split '$$'result " "] end]
if {{ '$$'keyword != \\"Complete!\\" }} {{
    exit 1
}}
$(TCL_CLOSE)'''

    TCL_CONTROLS = {'create': 'create_project $(PROJECT) ./',
                    'open': 'open_project $(PROJECT_FILE)',
                    'close': 'exit',
                    'project': '$(TCL_CREATE)\n'
                               '{0}\n'
                               'source files.tcl\n'
                               'update_compile_order -fileset sources_1\n'
                               'update_compile_order -fileset sim_1\n'
                               '$(TCL_CLOSE)',
                    'synthesize': _XILINX_RUN,
                    'par': _XILINX_RUN,
                    'install_source': '$(PROJECT).runs/impl_1/$(SYN_TOP).bit'}

    def __init__(self):
        super(ToolXilinx, self).__init__()
        self._hdl_files.update(ToolXilinx.HDL_FILES)
        self._supported_files.update(ToolXilinx.SUPPORTED_FILES)
        self._clean_targets.update(ToolXilinx.CLEAN_TARGETS)
        self._tcl_controls.update(ToolXilinx.TCL_CONTROLS)

    def _get_properties(self):
        """Create the property list"""
        syn_properties = self.manifest_dict.get("syn_properties")
        language = self.manifest_dict.get("language")
        if language == None:
            language = "vhdl"
        properties = [
            ['part', '$(SYN_DEVICE)' +
                     '$(SYN_PACKAGE)' +
                     '$(SYN_GRADE)', 'current_project'],
            ['target_language', language, 'current_project'],
            ['top', '$(TOP_MODULE)', 'get_property srcset [current_run]']]
        fetchto = self.manifest_dict.get("fetchto")
        if not fetchto is None:
            properties.append(['ip_repo_paths', fetchto, 'current_fileset'])
        if not syn_properties is None:
            properties.extend(syn_properties)
        return properties

    def _makefile_syn_tcl(self):
        """Create a Xilinx synthesis project by TCL"""
        prop_val = 'set_property "{0}" "{1}" [{2}]'
        prop_opt = 'set_property -name {{{0}}} -value {{{1}}} -objects [{2}]'
        project_new = ['# project properties']
        synthesize_new = ['# synthesize properties']
        par_new = ['# par properties']
        properties = self._get_properties()
        for prop in properties:
            if len(prop) > 1:
                tmp = prop_val
                name_list = prop[0].split()
                if len(name_list) == 2:
                    if name_list[1] == "options":
                        tmp = prop_opt
                    else:
                        logging.error('Unknown project property: %s', prop[0])
                if len(prop) == 2:
                    name_hierarchy = name_list[0].split(".")
                    if name_hierarchy[0] == "steps":
                        if name_hierarchy[1] == "synth_design":
                            synthesize_new.append(tmp.format(
                                prop[0], prop[1], 'get_runs synth_1'))
                        else:
                            par_new.append(tmp.format(
                                prop[0], prop[1], 'get_runs impl_1'))
                    else:
                        project_new.append(tmp.format(
                            prop[0], prop[1], 'current_project'))
                elif len(prop) == 3:
                    project_new.append(tmp.format(prop[0], prop[1], prop[2]))
                else:
                    logging.error('Unknown project property: %s', prop[0])
        tmp_dict = {}
        tmp_dict["project"] = self._tcl_controls["project"]
        tmp_dict["synthesize"] = self._tcl_controls["synthesize"]
        tmp_dict["par"] = self._tcl_controls["par"]
        self._tcl_controls["project"] = tmp_dict["project"].format(
            "\n".join(project_new))
        self._tcl_controls["synthesize"] = tmp_dict["synthesize"].format(
            "synth_1",
            "\n".join(synthesize_new))
        self._tcl_controls["par"] = tmp_dict["par"].format(
            "impl_1",
            "\n".join(par_new))
        super(ToolXilinx, self)._makefile_syn_tcl()



action = "simulation"
sim_tool = "active_hdl"
sim_top = "counter_tb"

sim_post_cmd = (
    "vsimsa -do ../play_sim.do\n\t\t"
    "avhdl wave.asdb")

modules = {
  "local" : [ "../../../testbench/counter_tb/vhdl" ],
}

